/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */
#include "adc.h"
#include "led.h"
#include <zephyr.h>
#include <sys/printk.h>
#include <drivers/pwm.h>

/*ADC definitions and includes*/
#include <hal/nrf_saadc.h>
//#define ADC_DEVICE_NAME DT_LABEL(DT_INST(0, nordic_nrf_saadc))
#define ADC_DEVICE_NAME DT_LABEL(DT_NODELABEL(adc0))
#define ADC_RESOLUTION 10	// 1024 = 1.2V, U = digits * 1.2 / 1024 * 6
#define ADC_GAIN ADC_GAIN_1_4  //ADC_GAIN_1_2
#define ADC_REFERENCE ADC_REF_INTERNAL
#define ADC_ACQUISITION_TIME ADC_ACQ_TIME(ADC_ACQ_TIME_MICROSECONDS, 10)
#define ADC_1ST_CHANNEL_ID 0  
#define ADC_1ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN0
#define ADC_2ND_CHANNEL_ID 1  
#define ADC_2ND_CHANNEL_INPUT NRF_SAADC_INPUT_AIN1
#define ADC_3RD_CHANNEL_ID 2  
#define ADC_3RD_CHANNEL_INPUT NRF_SAADC_INPUT_AIN2
#define ADC_4TH_CHANNEL_ID 3  
#define ADC_4TH_CHANNEL_INPUT NRF_SAADC_INPUT_AIN3

#define PWM_DEVICE_NAME DT_PROP(DT_NODELABEL(pwm0), label)
#define PWM_CH0_PIN DT_PROP(DT_NODELABEL(pwm0), ch0_pin)


#include <zephyr.h>
#include <device.h>
#include <drivers/gpio.h>
#include <drivers/adc.h>
#include <string.h>
#include <drivers/pwm.h>

// GPIO ports
//static const struct device *gpio_53_dev0;
//static const struct device *gpio_53_dev1;

#define PWM_MAX 253
#define TIMER_INTERVAL_MSEC 1000
#define BUFFER_SIZE 1

static float u_bat = 4.0;
static float u_usb = 5.0;
static float u_left = 0.14;
static float u_right = 0.14;
static int loops = 0;

struct k_timer my_timer;
const struct device *adc_dev;
//const struct device *pwm_dev;

static const struct adc_channel_cfg m_1st_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_1ST_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_1ST_CHANNEL_INPUT,
#endif
};

static const struct adc_channel_cfg m_2nd_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_2ND_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_2ND_CHANNEL_INPUT,
#endif
};

static const struct adc_channel_cfg m_3rd_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_3RD_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_3RD_CHANNEL_INPUT,
#endif
};

static const struct adc_channel_cfg m_4th_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_4TH_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_4TH_CHANNEL_INPUT,
#endif
};

static uint16_t m_sample_buffer[BUFFER_SIZE];

float get_u_bat(void)
{
	return u_bat;
}

float get_u_usb(void)
{
	return u_usb;
}


float get_u_left(void)
{
	return u_left;
}

float get_u_right(void)
{
	return u_right;
}

static int adc_sample_ain0(void)
{
	int ret;

	const struct adc_sequence sequence = {
		.channels = BIT(ADC_1ST_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	ret = adc_read(adc_dev, &sequence);
	if (ret) {
        printk("adc_read() failed with code %d\n", ret);
	}

	//for (int i = 0; i < BUFFER_SIZE; i++) {
				u_usb = (float)m_sample_buffer[0] * 1.2 / 1024.0 * 6.0;
				printk("USB: %d => %f\n", m_sample_buffer[0],u_usb);

				//printk("k-ADC raw value: %d\n", m_sample_buffer[i]);
                //float val = ((float)PWM_MAX/(float)568)*(float)m_sample_buffer[i];
                //printk("Setting pulse to: %f\n", val);
                //pwm_pin_set_usec(pwm_dev, PWM_CH0_PIN , PWM_MAX, val, 0);
	//}

	return ret;
}

static int adc_sample_ain1(void)
{
	int ret;

	const struct adc_sequence sequence = {
		.channels = BIT(ADC_2ND_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	ret = adc_read(adc_dev, &sequence);
	if (ret) {
        printk("adc_read() failed with code %d\n", ret);
	}

	u_bat = (float)m_sample_buffer[0] * 1.2 / 1024.0 * 6.0;
	printk("BAT: %d => %f\n", m_sample_buffer[0],u_bat);

	return ret;
}

static int adc_sample_ain2(void)
{
	int ret;

	const struct adc_sequence sequence = {
		.channels = BIT(ADC_3RD_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	ret = adc_read(adc_dev, &sequence);
	if (ret) {
        printk("adc_read() failed with code %d\n", ret);
	}

	u_left = (float)m_sample_buffer[0] * 1.2 / 1024.0;
	printk("LEFT: %d => %f\n", m_sample_buffer[0],u_left);

	return ret;
}

static int adc_sample_ain3(void)
{
	int ret;
	
	const struct adc_sequence sequence = {
		.channels = BIT(ADC_4TH_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	ret = adc_read(adc_dev, &sequence);
	if (ret) {
        printk("adc_read() failed with code %d\n", ret);
	}

	u_right = (float)m_sample_buffer[0] * 1.2 / 1024.0;
	printk("RIGHT: %d => %f\n", m_sample_buffer[0],u_right);

	return ret;
}

void adc_sample_event(struct k_timer *timer_id){
	static const struct device *gpio_53_dev1;
	int err;

	if (!adc_dev) {
		printk("no adc");
	}
	
    //err = adc_sample_ain0();
	//int ret;
    //if (err) {
    //    printk("Error in adc0 sampling: %d\n", err);
    //}
	//err = adc_sample_ain1();
    //if (err) {
    //    printk("Error in adc1 sampling: %d\n", err);
    //}
	if (loops >= 10) 
		{
		//ret = gpio_pin_configure(gpio_53_dev1, 14, GPIO_OUTPUT_HIGH);	// FET coil on
		//(void)k_sleep(K_MSEC(10));
		//err = adc_sample_ain2();
		if (err) {
			printk("Error in adc2 sampling: %d\n", err);
		}
		//ret = gpio_pin_configure(gpio_53_dev1, 14, GPIO_OUTPUT_LOW);	// FET coil off

		//ret = gpio_pin_configure(gpio_53_dev1, 11, GPIO_OUTPUT_HIGH);	// FET coil on
		//(void)k_sleep(K_MSEC(10));
		//err = adc_sample_ain3();
		if (err) {
			printk("Error in adc3 sampling: %d\n", err);
		}
		//ret = gpio_pin_configure(gpio_53_dev1, 11, GPIO_OUTPUT_LOW);	// FET coil off

		loops=0;
	}

	show_led_btle();

	loops++;
	
}

void init_adc(void)
{
    int err;

    //PWM0 setup
    //pwm_dev = device_get_binding(PWM_DEVICE_NAME);
    //if (!pwm_dev) {
	//    printk("device_get_binding() PWM0 failed\n");
	//}
    
     
    //ADC0 setup
	
    adc_dev = device_get_binding(ADC_DEVICE_NAME);
	
	if (!adc_dev) {
        printk("device_get_binding ADC_0 (=%s) failed\n", ADC_DEVICE_NAME);
    } 
    
    err = adc_channel_setup(adc_dev, &m_1st_channel_cfg);
    if (err) {
	    printk("Error in adc0 setup: %d\n", err);
	}
	err = adc_channel_setup(adc_dev, &m_2nd_channel_cfg);
    if (err) {
	    printk("Error in adc1 setup: %d\n", err);
	}
	err = adc_channel_setup(adc_dev, &m_3rd_channel_cfg);
    if (err) {
	    printk("Error in adc2 setup: %d\n", err);
	}
	err = adc_channel_setup(adc_dev, &m_4th_channel_cfg);
    if (err) {
	    printk("Error in adc3 setup: %d\n", err);
	}

	//Timer setup
    k_timer_init(&my_timer, adc_sample_event, NULL);
    k_timer_start(&my_timer, K_MSEC(TIMER_INTERVAL_MSEC), K_MSEC(TIMER_INTERVAL_MSEC));

}

